@@ -11,11 +11,40 @@ XXX: the whole mechanism should be fixed as now it's too complicated to explain
 from django.conf import settings
 
 
+class KeyMaker(object):
+    def __init__(self, prefix):
+        self.prefix = prefix
+
+    def convert(self, arg):
+        if isinstance(arg, unicode):
+            return arg.encode(settings.DEFAULT_CHARSET)
+        else:
+            return str(arg)
+
+    def args_to_key(self, args):
+        return ":".join(map(self.convert, args))
+
+    def kwargs_to_key(self, kwargs):
+        return ":".join([
+            "%s:%s" % (self.convert(k), self.convert(v))
+            for k, v in kwargs.items()
+        ])
+
+    def make(self, method_name, args, kwargs):
+        key = ":".join((
+            self.prefix,
+            method_name,
+            self.args_to_key(args),
+            self.kwargs_to_key(kwargs),
+        ))
+        return key
+
+
 class MethodProxy(object):
     def __init__(self, instance, method):
-        self.key_prefix = 'django_settings'
         self.instance = instance
         self.method = method  # accually it's NOT bounded s it's a function!
+        self._keymaker = KeyMaker(prefix='django_settings')
 
         # NOTE: it's proxy, so let's add at least some basic func properties
         self.func_name = self.method.func_name
@@ -27,47 +56,22 @@ class MethodProxy(object):
     def origin_method(self, *args, **kwargs):
         return self.method(self.instance, *args, **kwargs)
 
-    def _convert(self, arg):
-        if isinstance(arg, unicode):
-            return arg.encode(settings.DEFAULT_CHARSET)
-        else:
-            return str(arg)
+    def _cache_key(self, args, kwargs):
+        return self._keymaker.make(self.method.__name__, args, kwargs)
 
-    def _args_to_key(self, args):
-        return ":".join(map(self._convert, args))
-
-    def _kwargs_to_key(self, kwargs):
-        return ":".join([
-            "%s:%s" % (self._convert(k), self._convert(v))
-            for k, v in kwargs.items()
-        ])
-
-    def _cache_key_for_method(self, method_name, *args, **kwargs):
-        key = ":".join((
-            self.key_prefix,
-            method_name,
-            self._args_to_key(args),
-            self._kwargs_to_key(kwargs),
-        ))
-        return key
-
-    def _cache_key(self, *args, **kwargs):
-        return self._cache_key_for_method(self.method.__name__, *args, **kwargs)
-
-    def _cache_get(self, *args, **kwargs):
-        key = self._cache_key(*args, **kwargs)
+    def _cache_get(self, key):
         return self.cache.get(key)
 
-    def _cache_set(self, origin_value, *args, **kwargs):
-        key = self._cache_key(*args, **kwargs)
+    def _cache_set(self, key, origin_value):
         self.cache.set(key, origin_value)
         return origin_value
 
     def __call__(self, *args, **kwargs):
-        cached_value = self._cache_get(*args, **kwargs)
+        key = self._cache_key(args, kwargs)
+        cached_value = self._cache_get(key)
         if cached_value is None:
             origin_value = self.origin_method(*args, **kwargs)
-            cached_value = self._cache_set(origin_value, *args, **kwargs)
+            cached_value = self._cache_set(key, origin_value)
         return cached_value
 
 
